﻿using System;
using System.Drawing;

namespace EsempioDecorator
{
    public interface IVisitor
    {
        void Visit(TextFrame component);
        void Visit(BorderedFrame component);
        void Visit(FilledFrame component);
        void Visit(RotatedFrame component);
    }

    public class XMLVisitor : IVisitor
    {
        private string _xmlString = "";

        public string XMLString
        {
            get { return _xmlString; }
        }

        public void Visit(TextFrame component)
        {
            BeginTag("TextFrame");
            AddToXML(component.Text);
            EndTag("TextFrame");
        }

        public void Visit(BorderedFrame decorator)
        {
            BeginTag("BorderedFrame");
            decorator.Component.Accept(this);
            EndTag("BorderedFrame");
        }

        public void Visit(FilledFrame decorator)
        {
            BeginTag("FilledFrame");
            decorator.Component.Accept(this);
            EndTag("FilledFrame");
        }

        public void Visit(RotatedFrame decorator)
        {
            BeginTag("RotatedFrame");
            decorator.Component.Accept(this);
            EndTag("RotatedFrame");
        }

        private int _ident = -1;

        private void BeginTag(string tagName)
        {
            _ident++;
            AddToXML("<" + tagName + ">");
        }

        private void EndTag(string tagName)
        {
            AddToXML("</" + tagName + ">");
            _ident--;
        }

        private void AddToXML(string text)
        {
            if (_ident > 0)
                _xmlString += new String(' ', _ident * 3);
            _xmlString += text + "\n";
        }
    }

    public class BorderVisitor : IVisitor
    {
        public void Visit(TextFrame component)
        {
        }

        public void Visit(BorderedFrame decorator)
        {
            decorator.Pen = new Pen(decorator.Pen.Brush, decorator.Pen.Width + 1);
            decorator.Component.Accept(this);
        }

        public void Visit(FilledFrame decorator)
        {
            decorator.Component.Accept(this);
        }

        public void Visit(RotatedFrame decorator)
        {
            decorator.Component.Accept(this);
        }
    }
}
